@@ -21,9 +21,7 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "evolution-ews-config.h"
 
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
@@ -362,7 +360,7 @@ search_term_changed_cb (GtkEntry *entry,
 		sid->cancellable = g_object_ref (pgu->cancellable);
 		sid->dialog = dialog;
 
-		gtk_label_set_text (GTK_LABEL (pgu->info_label), _("Searching..."));
+		gtk_label_set_text (GTK_LABEL (pgu->info_label), _("Searching…"));
 		pgu->schedule_search_id = e_named_timeout_add (333, schedule_search_cb, sid);
 	}
 }
@@ -415,6 +413,7 @@ create_users_tree_view (GtkWidget *dialog,
                         struct EEwsSearchUserData *pgu)
 {
 	GtkTreeView *tree_view;
+	GtkTreeModel *model;
 	GtkTreeSelection *selection;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
@@ -423,8 +422,9 @@ create_users_tree_view (GtkWidget *dialog,
 	g_return_val_if_fail (dialog != NULL, NULL);
 	g_return_val_if_fail (pgu != NULL, NULL);
 
-	tree_view = GTK_TREE_VIEW (gtk_tree_view_new_with_model (
-		GTK_TREE_MODEL (gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_UINT))));
+	model = GTK_TREE_MODEL (gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_UINT));
+	tree_view = GTK_TREE_VIEW (gtk_tree_view_new_with_model (model));
+	g_object_unref (model);
 
 	renderer = gtk_cell_renderer_text_new ();
 	g_object_set (renderer, "editable", FALSE, NULL);
@@ -471,7 +471,7 @@ e_ews_search_user_modal (GtkWindow *parent,
 	pgu->conn = g_object_ref (conn);
 
 	dialog = gtk_dialog_new_with_buttons (
-		_("Choose EWS user..."),
+		_("Choose EWS user…"),
 		parent,
 		GTK_DIALOG_DESTROY_WITH_PARENT,
 		GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
