@@ -403,7 +403,7 @@ e_ews_cal_utils_set_time (ESoapMessage *msg,
 	g_return_if_fail (tt != NULL);
 
 	if (with_timezone) {
-		if (icaltime_is_utc (*tt) || !tt->zone || tt->zone == icaltimezone_get_utc_timezone ()) {
+		if (icaltime_is_utc (*tt) || !tt->zone || tt->zone == icaltimezone_get_utc_timezone () || tt->is_date) {
 			tz_ident = g_strdup ("Z");
 		} else {
 			gint offset, is_daylight, hrs, mins;
